export interface MaterialExportAsNameData {
    /** The exportAs name to replace. */
    replace: string;
    /** The new exportAs name. */
    replaceWith: string;
}
export interface MaterialElementSelectorData {
    /** The element name to replace. */
    replace: string;
    /** The new name for the element. */
    replaceWith: string;
}
export interface MaterialCssNameData {
    /** The CSS name to replace. */
    replace: string;
    /** The new CSS name. */
    replaceWith: string;
    /** Whitelist where this replacement is made. If omitted it is made in all files. */
    whitelist: {
        /** Replace this name in CSS files. */
        css?: boolean;
        /** Replace this name in HTML files. */
        html?: boolean;
        /** Replace this name in TypeScript strings. */
        strings?: boolean;
    };
}
export interface MaterialAttributeSelectorData {
    /** The attribute name to replace. */
    replace: string;
    /** The new name for the attribute. */
    replaceWith: string;
}
export interface MaterialPropertyNameData {
    /** The property name to replace. */
    replace: string;
    /** The new name for the property. */
    replaceWith: string;
    /** Whitelist where this replacement is made. If omitted it is made for all Classes. */
    whitelist: {
        /** Replace the property only when its type is one of the given Classes. */
        classes?: string[];
    };
}
export interface MaterialClassNameData {
    /** The Class name to replace. */
    replace: string;
    /** The new name for the Class. */
    replaceWith: string;
}
export interface MaterialInputNameData {
    /** The @Input() name to replace. */
    replace: string;
    /** The new name for the @Input(). */
    replaceWith: string;
    /** Whitelist where this replacement is made. If omitted it is made in all HTML & CSS */
    whitelist?: {
        /** Limit to elements with any of these element tags. */
        elements?: string[];
        /** Limit to elements with any of these attributes. */
        attributes?: string[];
        /** Whether to ignore CSS attribute selectors when doing this replacement. */
        css?: boolean;
    };
}
export interface MaterialOutputNameData {
    /** The @Output() name to replace. */
    replace: string;
    /** The new name for the @Output(). */
    replaceWith: string;
    /** Whitelist where this replacement is made. If omitted it is made in all HTML & CSS */
    whitelist?: {
        /** Limit to elements with any of these element tags. */
        elements?: string[];
        /** Limit to elements with any of these attributes. */
        attributes?: string[];
        /** Whether to ignore CSS attribute selectors when doing this replacement. */
        css?: boolean;
    };
}
export interface MaterialMethodCallData {
    className: string;
    method: string;
    invalidArgCounts: {
        count: number;
        message: string;
    }[];
}
/** Export the class name data as part of a module. This means that the data is cached. */
export declare const classNames: MaterialClassNameData[];
/** Export the input names data as part of a module. This means that the data is cached. */
export declare const inputNames: MaterialInputNameData[];
/** Export the output names data as part of a module. This means that the data is cached. */
export declare const outputNames: MaterialOutputNameData[];
/** Export the element selectors data as part of a module. This means that the data is cached. */
export declare const elementSelectors: MaterialElementSelectorData[];
/** Export the attribute selectors data as part of a module. This means that the data is cached. */
export declare const exportAsNames: MaterialExportAsNameData[];
/** Export the attribute selectors data as part of a module. This means that the data is cached. */
export declare const attributeSelectors: MaterialAttributeSelectorData[];
/** Export the property names as part of a module. This means that the data is cached. */
export declare const propertyNames: MaterialPropertyNameData[];
export declare const methodCallChecks: MaterialMethodCallData[];
export declare const cssNames: MaterialCssNameData[];
