"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var component_data_1 = require("../material/component-data");
/**
 * Rule that walks through every property access expression and updates properties that have
 * been changed in favor of the new name.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.applyWithProgram = function (sourceFile, program) {
        return this.applyWithWalker(new CheckInheritanceWalker(sourceFile, this.getOptions(), program));
    };
    return Rule;
}(tslint_1.Rules.TypedRule));
exports.Rule = Rule;
var CheckInheritanceWalker = (function (_super) {
    __extends(CheckInheritanceWalker, _super);
    function CheckInheritanceWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CheckInheritanceWalker.prototype.visitClassDeclaration = function (declaration) {
        var _this = this;
        // Check if user is extending an Angular Material class whose properties have changed.
        var type = this.getTypeChecker().getTypeAtLocation(declaration.name);
        var baseTypes = this.getTypeChecker().getBaseTypes(type);
        baseTypes.forEach(function (t) {
            var propertyData = component_data_1.propertyNames.find(function (data) { return data.whitelist && new Set(data.whitelist.classes).has(t.symbol.name); });
            if (propertyData) {
                _this.addFailureAtNode(declaration, "Found class \"" + chalk_1.bold(declaration.name.text) + "\" which extends class" +
                    (" \"" + chalk_1.bold(t.symbol.name) + "\". Please note that the base class property") +
                    (" \"" + chalk_1.red(propertyData.replace) + "\" has changed to \"" + chalk_1.green(propertyData.replaceWith) + "\".") +
                    " You may need to update your class as well");
            }
        });
    };
    return CheckInheritanceWalker;
}(tslint_1.ProgramAwareRuleWalker));
exports.CheckInheritanceWalker = CheckInheritanceWalker;
//# sourceMappingURL=checkInheritanceRule.js.map