"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var ts = require("typescript");
var color_1 = require("../material/color");
var component_data_1 = require("../material/component-data");
/**
 * Rule that walks through every property access expression and updates properties that have
 * been changed in favor of the new name.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.applyWithProgram = function (sourceFile, program) {
        return this.applyWithWalker(new CheckMethodCallsWalker(sourceFile, this.getOptions(), program));
    };
    return Rule;
}(tslint_1.Rules.TypedRule));
exports.Rule = Rule;
var CheckMethodCallsWalker = (function (_super) {
    __extends(CheckMethodCallsWalker, _super);
    function CheckMethodCallsWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CheckMethodCallsWalker.prototype.visitNewExpression = function (expression) {
        var className = this.getTypeChecker().getTypeAtLocation(expression).symbol.name;
        this.checkConstructor(expression, className);
    };
    CheckMethodCallsWalker.prototype.visitCallExpression = function (expression) {
        if (expression.expression.kind !== ts.SyntaxKind.PropertyAccessExpression) {
            var methodName_1 = expression.getFirstToken().getText();
            if (methodName_1 === 'super') {
                var type_1 = this.getTypeChecker().getTypeAtLocation(expression.expression);
                var className_1 = type_1.symbol && type_1.symbol.name;
                this.checkConstructor(expression, className_1);
            }
            return;
        }
        // TODO(mmalerba): This is probably a bad way to get the class node...
        // Tokens are: [..., <host>, '.', <prop>], so back up 3.
        var accessExp = expression.expression;
        var classNode = accessExp.getChildAt(accessExp.getChildCount() - 3);
        var methodNode = accessExp.getChildAt(accessExp.getChildCount() - 1);
        var methodName = methodNode.getText();
        var type = this.getTypeChecker().getTypeAtLocation(classNode);
        var className = type.symbol && type.symbol.name;
        var currentCheck = component_data_1.methodCallChecks
            .find(function (data) { return data.method === methodName && data.className === className; });
        if (!currentCheck) {
            return;
        }
        var failure = currentCheck.invalidArgCounts
            .find(function (countData) { return countData.count === expression.arguments.length; });
        if (failure) {
            this.addFailureAtNode(expression, "Found call to \"" + chalk_1.bold(className + '.' + methodName) + "\" with" +
                (" " + chalk_1.bold(String(failure.count)) + " arguments. " + color_1.color(failure.message)));
        }
    };
    CheckMethodCallsWalker.prototype.checkConstructor = function (node, className) {
        var currentCheck = component_data_1.methodCallChecks
            .find(function (data) { return data.method === 'constructor' && data.className === className; });
        if (!currentCheck) {
            return;
        }
        var failure = currentCheck.invalidArgCounts
            .find(function (countData) { return countData.count === node.arguments.length; });
        if (failure) {
            this.addFailureAtNode(node, "Found \"" + chalk_1.bold(className) + "\" constructed with " + chalk_1.bold(String(failure.count)) + " arguments." +
                (" " + color_1.color(failure.message)));
        }
    };
    return CheckMethodCallsWalker;
}(tslint_1.ProgramAwareRuleWalker));
exports.CheckMethodCallsWalker = CheckMethodCallsWalker;
//# sourceMappingURL=checkMethodCallsRule.js.map