"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var ts = require("typescript");
/**
 * Rule that walks through every identifier that is part of Angular Material and replaces the
 * outdated name with the new one.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.applyWithProgram = function (sourceFile, program) {
        return this.applyWithWalker(new CheckPropertyAccessMiscWalker(sourceFile, this.getOptions(), program));
    };
    return Rule;
}(tslint_1.Rules.TypedRule));
exports.Rule = Rule;
var CheckPropertyAccessMiscWalker = (function (_super) {
    __extends(CheckPropertyAccessMiscWalker, _super);
    function CheckPropertyAccessMiscWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CheckPropertyAccessMiscWalker.prototype.visitPropertyAccessExpression = function (prop) {
        // Recursively call this method for the expression of the current property expression.
        // It can happen that there is a chain of property access expressions.
        // For example: "mySortInstance.mdSortChange.subscribe()"
        if (prop.expression && prop.expression.kind === ts.SyntaxKind.PropertyAccessExpression) {
            this.visitPropertyAccessExpression(prop.expression);
        }
        // TODO(mmalerba): This is prrobably a bad way to get the property host...
        // Tokens are: [..., <host>, '.', <prop>], so back up 3.
        var propHost = prop.getChildAt(prop.getChildCount() - 3);
        var type = this.getTypeChecker().getTypeAtLocation(propHost);
        var typeName = type && type.getSymbol() && type.getSymbol().getName();
        if (typeName === 'MatListOption' && prop.name.text === 'selectionChange') {
            this.addFailureAtNode(prop, "Found deprecated property \"" + chalk_1.red('selectionChange') + "\" of class" +
                (" \"" + chalk_1.bold('MatListOption') + "\". Use the \"" + chalk_1.green('selectionChange') + "\" property on the") +
                (" parent \"" + chalk_1.bold('MatSelectionList') + "\" instead."));
        }
        if (typeName === 'MatDatepicker' && prop.name.text === 'selectedChanged') {
            this.addFailureAtNode(prop, "Found deprecated property \"" + chalk_1.red('selectedChanged') + "\" of class" +
                (" \"" + chalk_1.bold('MatDatepicker') + "\". Use the \"" + chalk_1.green('dateChange') + "\" or") +
                (" \"" + chalk_1.green('dateInput') + "\" methods on \"" + chalk_1.bold('MatDatepickerInput') + "\" instead"));
        }
    };
    return CheckPropertyAccessMiscWalker;
}(tslint_1.ProgramAwareRuleWalker));
exports.CheckPropertyAccessMiscWalker = CheckPropertyAccessMiscWalker;
//# sourceMappingURL=checkPropertyAccessMiscRule.js.map