"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var component_walker_1 = require("../tslint/component-walker");
var literal_1 = require("../typescript/literal");
/**
 * Rule that walks through every component decorator and updates their inline or external
 * templates.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        return this.applyWithWalker(new CheckTemplateMiscWalker(sourceFile, this.getOptions()));
    };
    return Rule;
}(tslint_1.Rules.AbstractRule));
exports.Rule = Rule;
var CheckTemplateMiscWalker = (function (_super) {
    __extends(CheckTemplateMiscWalker, _super);
    function CheckTemplateMiscWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CheckTemplateMiscWalker.prototype.visitInlineTemplate = function (template) {
        var _this = this;
        this.checkTemplate(template, template.getText()).forEach(function (failure) {
            var ruleFailure = new tslint_1.RuleFailure(template.getSourceFile(), failure.start, failure.end, failure.message, _this.getRuleName());
            _this.addFailure(ruleFailure);
        });
    };
    CheckTemplateMiscWalker.prototype.visitExternalTemplate = function (template) {
        var _this = this;
        this.checkTemplate(template, template.getFullText()).forEach(function (failure) {
            var ruleFailure = new tslint_1.RuleFailure(template, failure.start + 1, failure.end + 1, failure.message, _this.getRuleName());
            _this.addFailure(ruleFailure);
        });
    };
    /**
     * Replaces the outdated name in the template with the new one and returns an updated template.
     */
    CheckTemplateMiscWalker.prototype.checkTemplate = function (node, templateContent) {
        var failures = [];
        failures = failures.concat(literal_1.findAll(templateContent, 'cdk-focus-trap').map(function (offset) { return ({
            start: offset,
            end: offset + 'cdk-focus-trap'.length,
            message: "Found deprecated element selector \"" + chalk_1.red('cdk-focus-trap') + "\" which has been" +
                (" changed to an attribute selector \"" + chalk_1.green('[cdkTrapFocus]') + "\"")
        }); }));
        failures = failures.concat(literal_1.findAllOutputsInElWithTag(templateContent, 'selectionChange', ['mat-list-option'])
            .map(function (offset) { return ({
            start: offset,
            end: offset + 'selectionChange'.length,
            message: "Found deprecated @Output() \"" + chalk_1.red('selectionChange') + "\" on" +
                (" \"" + chalk_1.bold('mat-list-option') + "\". Use \"" + chalk_1.green('selectionChange') + "\" on") +
                (" \"" + chalk_1.bold('mat-selection-list') + "\" instead")
        }); }));
        failures = failures.concat(literal_1.findAllOutputsInElWithTag(templateContent, 'selectedChanged', ['mat-datepicker'])
            .map(function (offset) { return ({
            start: offset,
            end: offset + 'selectionChange'.length,
            message: "Found deprecated @Output() \"" + chalk_1.red('selectedChanged') + "\" on" +
                (" \"" + chalk_1.bold('mat-datepicker') + "\". Use \"" + chalk_1.green('dateChange') + "\" or") +
                (" \"" + chalk_1.green('dateInput') + "\" on \"" + chalk_1.bold('<input [matDatepicker]>') + "\" instead")
        }); }));
        failures = failures.concat(literal_1.findAllInputsInElWithTag(templateContent, 'selected', ['mat-button-toggle-group'])
            .map(function (offset) { return ({
            start: offset,
            end: offset + 'selected'.length,
            message: "Found deprecated @Input() \"" + chalk_1.red('selected') + "\" on" +
                (" \"" + chalk_1.bold('mat-radio-button-group') + "\". Use \"" + chalk_1.green('value') + "\" instead")
        }); }));
        return failures;
    };
    return CheckTemplateMiscWalker;
}(component_walker_1.ComponentWalker));
exports.CheckTemplateMiscWalker = CheckTemplateMiscWalker;
//# sourceMappingURL=checkTemplateMiscRule.js.map