"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var ts = require("typescript");
var component_data_1 = require("../material/component-data");
/**
 * Rule that walks through every property access expression and updates properties that have
 * been changed in favor of the new name.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.applyWithProgram = function (sourceFile, program) {
        return this.applyWithWalker(new SwitchPropertyNamesWalker(sourceFile, this.getOptions(), program));
    };
    return Rule;
}(tslint_1.Rules.TypedRule));
exports.Rule = Rule;
var SwitchPropertyNamesWalker = (function (_super) {
    __extends(SwitchPropertyNamesWalker, _super);
    function SwitchPropertyNamesWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SwitchPropertyNamesWalker.prototype.visitPropertyAccessExpression = function (prop) {
        // Recursively call this method for the expression of the current property expression.
        // It can happen that there is a chain of property access expressions.
        // For example: "mySortInstance.mdSortChange.subscribe()"
        if (prop.expression && prop.expression.kind === ts.SyntaxKind.PropertyAccessExpression) {
            this.visitPropertyAccessExpression(prop.expression);
        }
        // TODO(mmalerba): This is prrobably a bad way to get the property host...
        // Tokens are: [..., <host>, '.', <prop>], so back up 3.
        var propHost = prop.getChildAt(prop.getChildCount() - 3);
        var type = this.getTypeChecker().getTypeAtLocation(propHost);
        var typeName = type && type.getSymbol() && type.getSymbol().getName();
        var propertyData = component_data_1.propertyNames.find(function (name) {
            if (prop.name.text === name.replace) {
                // TODO(mmalerba): Verify that this type comes from Angular Material like we do in
                // `switchIdentifiersRule`.
                return !name.whitelist || new Set(name.whitelist.classes).has(typeName);
            }
            return false;
        });
        if (!propertyData) {
            return;
        }
        var replacement = this.createReplacement(prop.name.getStart(), prop.name.getWidth(), propertyData.replaceWith);
        var typeMessage = propertyData.whitelist ? "of class \"" + chalk_1.bold(typeName) + "\"" : '';
        this.addFailureAtNode(prop.name, "Found deprecated property \"" + chalk_1.red(propertyData.replace) + "\" " + typeMessage + " which has been" +
            (" renamed to \"" + chalk_1.green(propertyData.replaceWith) + "\""), replacement);
    };
    return SwitchPropertyNamesWalker;
}(tslint_1.ProgramAwareRuleWalker));
exports.SwitchPropertyNamesWalker = SwitchPropertyNamesWalker;
//# sourceMappingURL=switchPropertyNamesRule.js.map