"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var ts = require("typescript");
var component_data_1 = require("../material/component-data");
var literal_1 = require("../typescript/literal");
/**
 * Rule that walks through every string literal, which includes the outdated Material name and
 * is part of a call expression. Those string literals will be changed to the new name.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        return this.applyWithWalker(new SwitchStringLiteralCssNamesWalker(sourceFile, this.getOptions()));
    };
    return Rule;
}(tslint_1.Rules.AbstractRule));
exports.Rule = Rule;
var SwitchStringLiteralCssNamesWalker = (function (_super) {
    __extends(SwitchStringLiteralCssNamesWalker, _super);
    function SwitchStringLiteralCssNamesWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SwitchStringLiteralCssNamesWalker.prototype.visitStringLiteral = function (stringLiteral) {
        var _this = this;
        if (stringLiteral.parent.kind !== ts.SyntaxKind.CallExpression) {
            return;
        }
        var stringLiteralText = stringLiteral.getFullText();
        component_data_1.cssNames.forEach(function (name) {
            if (!name.whitelist || name.whitelist.strings) {
                _this.createReplacementsForOffsets(stringLiteral, name, literal_1.findAll(stringLiteralText, name.replace)).forEach(function (replacement) {
                    _this.addFailureAtNode(stringLiteral, "Found deprecated CSS class \"" + chalk_1.red(name.replace) + "\" which has been renamed to" +
                        (" \"" + chalk_1.green(name.replaceWith) + "\""), replacement);
                });
            }
        });
    };
    SwitchStringLiteralCssNamesWalker.prototype.createReplacementsForOffsets = function (node, update, offsets) {
        var _this = this;
        return offsets.map(function (offset) { return _this.createReplacement(node.getStart() + offset, update.replace.length, update.replaceWith); });
    };
    return SwitchStringLiteralCssNamesWalker;
}(tslint_1.RuleWalker));
exports.SwitchStringLiteralCssNamesWalker = SwitchStringLiteralCssNamesWalker;
//# sourceMappingURL=switchStringLiteralCssNamesRule.js.map