"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var tslint_1 = require("tslint");
var ts = require("typescript");
var component_data_1 = require("../material/component-data");
var literal_1 = require("../typescript/literal");
/**
 * Rule that walks through every string literal, which includes the outdated Material name and
 * is part of a call expression. Those string literals will be changed to the new name.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        return this.applyWithWalker(new SwitchStringLiteralElementSelectorsWalker(sourceFile, this.getOptions()));
    };
    return Rule;
}(tslint_1.Rules.AbstractRule));
exports.Rule = Rule;
var SwitchStringLiteralElementSelectorsWalker = (function (_super) {
    __extends(SwitchStringLiteralElementSelectorsWalker, _super);
    function SwitchStringLiteralElementSelectorsWalker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SwitchStringLiteralElementSelectorsWalker.prototype.visitStringLiteral = function (stringLiteral) {
        var _this = this;
        if (stringLiteral.parent.kind !== ts.SyntaxKind.CallExpression) {
            return;
        }
        var stringLiteralText = stringLiteral.getFullText();
        component_data_1.elementSelectors.forEach(function (selector) {
            _this.createReplacementsForOffsets(stringLiteral, selector, literal_1.findAll(stringLiteralText, selector.replace)).forEach(function (replacement) {
                _this.addFailureAtNode(stringLiteral, "Found deprecated element selector \"" + chalk_1.red(selector.replace) + "\" which has been" +
                    (" renamed to \"" + chalk_1.green(selector.replaceWith) + "\""), replacement);
            });
        });
    };
    SwitchStringLiteralElementSelectorsWalker.prototype.createReplacementsForOffsets = function (node, update, offsets) {
        var _this = this;
        return offsets.map(function (offset) { return _this.createReplacement(node.getStart() + offset, update.replace.length, update.replaceWith); });
    };
    return SwitchStringLiteralElementSelectorsWalker;
}(tslint_1.RuleWalker));
exports.SwitchStringLiteralElementSelectorsWalker = SwitchStringLiteralElementSelectorsWalker;
//# sourceMappingURL=switchStringLiteralElementSelectorsRule.js.map