"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var glob_1 = require("glob");
var path_1 = require("path");
var tslint_1 = require("tslint");
var component_data_1 = require("../material/component-data");
var extra_stylsheets_1 = require("../material/extra-stylsheets");
var component_walker_1 = require("../tslint/component-walker");
var literal_1 = require("../typescript/literal");
/**
 * Rule that walks through every component decorator and updates their inline or external
 * stylesheets.
 */
var Rule = (function (_super) {
    __extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (sourceFile) {
        return this.applyWithWalker(new SwitchStylesheetElementSelectorsWalker(sourceFile, this.getOptions()));
    };
    return Rule;
}(tslint_1.Rules.AbstractRule));
exports.Rule = Rule;
var SwitchStylesheetElementSelectorsWalker = (function (_super) {
    __extends(SwitchStylesheetElementSelectorsWalker, _super);
    function SwitchStylesheetElementSelectorsWalker(sourceFile, options) {
        var _this = this;
        // This is a special feature. In some applications, developers will have global stylesheets
        // that are not specified in any Angular component. Those stylesheets can be also migrated
        // if the developer specifies the `--extra-stylesheets` option which accepts a glob for files.
        var extraFiles = [];
        if (process.env[extra_stylsheets_1.EXTRA_STYLESHEETS_GLOB_KEY]) {
            process.env[extra_stylsheets_1.EXTRA_STYLESHEETS_GLOB_KEY].split(' ')
                .map(function (glob) { return glob_1.sync(glob); })
                .forEach(function (files) { return files.forEach(function (styleUrl) {
                extraFiles.push(path_1.resolve(styleUrl));
            }); });
        }
        _this = _super.call(this, sourceFile, options, extraFiles) || this;
        extraFiles.forEach(function (styleUrl) { return _this._reportExternalStyle(styleUrl); });
        return _this;
    }
    SwitchStylesheetElementSelectorsWalker.prototype.visitInlineStylesheet = function (stylesheet) {
        var _this = this;
        this.replaceNamesInStylesheet(stylesheet, stylesheet.getText()).forEach(function (replacement) {
            var fix = replacement.replacement;
            var ruleFailure = new tslint_1.RuleFailure(stylesheet.getSourceFile(), fix.start, fix.end, replacement.message, _this.getRuleName(), fix);
            _this.addFailure(ruleFailure);
        });
    };
    SwitchStylesheetElementSelectorsWalker.prototype.visitExternalStylesheet = function (stylesheet) {
        var _this = this;
        this.replaceNamesInStylesheet(stylesheet, stylesheet.getFullText()).forEach(function (replacement) {
            var fix = replacement.replacement;
            var ruleFailure = new tslint_1.RuleFailure(stylesheet, fix.start + 1, fix.end + 1, replacement.message, _this.getRuleName(), fix);
            _this.addFailure(ruleFailure);
        });
    };
    /**
     * Replaces the outdated name in the stylesheet with the new one and returns an updated
     * stylesheet.
     */
    SwitchStylesheetElementSelectorsWalker.prototype.replaceNamesInStylesheet = function (node, stylesheetContent) {
        var _this = this;
        var replacements = [];
        component_data_1.elementSelectors.forEach(function (selector) {
            _this.createReplacementsForOffsets(node, selector, literal_1.findAll(stylesheetContent, selector.replace)).forEach(function (replacement) {
                replacements.push({
                    message: "Found deprecated element selector \"" + chalk_1.red(selector.replace) + "\" which has" +
                        (" been renamed to \"" + chalk_1.green(selector.replaceWith) + "\""),
                    replacement: replacement
                });
            });
        });
        return replacements;
    };
    SwitchStylesheetElementSelectorsWalker.prototype.createReplacementsForOffsets = function (node, update, offsets) {
        var _this = this;
        return offsets.map(function (offset) { return _this.createReplacement(node.getStart() + offset, update.replace.length, update.replaceWith); });
    };
    return SwitchStylesheetElementSelectorsWalker;
}(component_walker_1.ComponentWalker));
exports.SwitchStylesheetElementSelectorsWalker = SwitchStylesheetElementSelectorsWalker;
//# sourceMappingURL=switchStylesheetElementSelectorsRule.js.map